﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.IO;
using System.Runtime.Serialization;
using DlhSoft.Windows.Data;

namespace GanttChartTaskListViewXmlFileSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();

            ObservableCollection<TaskItem> taskItems = new ObservableCollection<TaskItem>
            {
                new TaskItem { Content = "Task 1", IndexString = "1" },
                new TaskItem { Content = "Task 1.1", Indentation = 1, Start = DateTime.Today, Effort = TimeSpan.Parse("08:00:00"), CompletedEffort = TimeSpan.Parse("04:00:00"), IndexString = "2" },
                new TaskItem { Content = "Task 1.2", Indentation = 1, IndexString = "3" },
                new TaskItem { Content = "Task 1.2.1", Indentation = 2, Start = DateTime.Today, Effort = TimeSpan.Parse("08:00:00"), IndexString = "4" },
                new TaskItem { Content = "Task 1.2.2", Indentation = 2, Start = DateTime.Today, Effort = TimeSpan.Parse("16:00:00"), IndexString = "5" },
                new TaskItem { Content = "Task 2", IsExpanded = true, IndexString = "6" },
                new TaskItem { Content = "Task 2.1", Indentation = 1, Start = DateTime.Today, Effort = TimeSpan.Parse("16:00:00"), IsCompleted = true, IndexString = "7" },
                new TaskItem { Content = "Task 2.2", Indentation = 1, Start = DateTime.Today, Effort = TimeSpan.Parse("08:00:00"), CompletedEffort = TimeSpan.Parse("06:00:00"), IndexString = "8" },
                new TaskItem { Content = "Task 2.3", Indentation = 1, IsExpanded = true, IndexString = "9", PredecessorsString = "8" },
                new TaskItem { Content = "Task 2.3.1", Indentation = 2, Start = DateTime.Today, Effort = TimeSpan.Parse("1.16:00:00"), IndexString = "10", Cost = 2000 },
                new TaskItem { Content = "Task 2.3.2", Indentation = 2, Start = DateTime.Today, Effort = TimeSpan.Parse("08:00:00"), IndexString = "11", PredecessorsString = "10" },
                new TaskItem { Content = "Task 3", Start = DateTime.Today, Effort = TimeSpan.Parse("12:30:00"), CompletedEffort = TimeSpan.Parse("02:30:00"), IndexString = "12", PredecessorsString = "9, 11SS+12" },
                new TaskItem { Content = "Task 4", Start = DateTime.Today, Effort = TimeSpan.Parse("16:00:00"), AssignmentsString = "Resource 1", IndexString = "13" },
                new TaskItem { Content = "Task 5", Start = DateTime.Today, Effort = TimeSpan.Parse("20:00:00"), AssignmentsString = "Resource 1, Resource 2 [50%]", IndexString = "14" },
                new TaskItem { Content = "Task 6", Start = DateTime.Today, IsMilestone = true, IndexString = "15", PredecessorsString = "14+2" }
            };

            ObservableCollection<ResourceItem> assignableResourceItems = new ObservableCollection<ResourceItem>
            {
                new ResourceItem { Content = "Resource 1", CostPerUsage = 100, CostPerHour = 10 },
                new ResourceItem { Content = "Resource 2", CostPerHour = 20 }
            };

            GanttChartTaskListView.AssignableResourceItemsSource = assignableResourceItems;
            GanttChartTaskListView.ItemsSource = taskItems;
        }

        private void LoadMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Load task data from an XML file.
            DataContractSerializer serializer = new DataContractSerializer(typeof(TaskItem[]));
            try
            {
                using (FileStream fileStream = new FileStream("Tasks.xml", FileMode.Open))
                {
                    TaskItem[] taskItemsArray = serializer.ReadObject(fileStream) as TaskItem[];
                    ObservableCollection<TaskItem> taskItems = new ObservableCollection<TaskItem>(taskItemsArray.ToList());
                    GanttChartTaskListView.ItemsSource = taskItems;
                }
            }
            catch (IOException)
            {
                MessageBox.Show("Cannot load data from file Tasks.xml.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            catch (SerializationException)
            {
                MessageBox.Show("Cannot read data from file Tasks.xml (it may have invalid content).", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            
            // Scroll the view to properly display the project.
            DateTime overallStart = GanttChartTaskListView.OverallStart;
            DateTime overallFinish = GanttChartTaskListView.OverallFinish;
            DateTime now = DateTime.Now;
            GanttChartTaskListView.SetTimelinePage(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) : now.AddDays(-1)), GanttChartTaskListView.TimelinePageDuration);
            GanttChartTaskListView.ScrollTo(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1) : now.AddDays(-1)));
        }

        private void SaveMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Save task data to an XML file.
            DataContractSerializer serializer = new DataContractSerializer(typeof(TaskItem[]));
            try
            {
                using (FileStream fileStream = new FileStream("Tasks.xml", FileMode.Create))
                {
                    ObservableCollection<TaskItem> taskItems = GanttChartTaskListView.ItemsSource as ObservableCollection<TaskItem>;
                    TaskItem[] taskItemsArray = taskItems.ToArray();
                    serializer.WriteObject(fileStream, taskItemsArray);
                }
            }
            catch (IOException)
            {
                MessageBox.Show("Cannot save data to file Tasks.xml.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
        }
    }
}
